/*------------------------------------------------------------------------------
 Copyright:      Markus Kotschenreuther
 Author:         Markus Kotschenreuther
 Remarks:
 known Problems: none
 Version:        25.04.2009
 Description:    basic control routines

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der
 GNU General Public License, wie von der Free Software Foundation verffentlicht,
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder
 (nach Ihrer Option) jeder spteren Version.

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung,
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE,
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License.

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
 Programm erhalten haben.
 Falls nicht, schreiben Sie an die Free Software Foundation,
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
------------------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "hp03.h"
#include "digital_io.h"
#include "std_defs.h"
#include <avr/io.h>
#include <avr/interrupt.h>

#include <twimaster.c>

//todo testonly
#include <stdlib.h>
#include <uart.h>

/************************* defines ******************************************/
#define EEPROM_ADDR  	0xA2
#define MEASVAL_ADDR  	0xEE

// defines the data direction (reading from I2C device)
#define I2C_READ    1

// defines the data direction (writing to I2C device)
#define I2C_WRITE   0

/************************* typedefs ******************************************/

/************************* globals ******************************************/
unsigned int uiCoeff[7];
unsigned char ucParam[4];

/************************* function prototypes ******************************/


//############################################################################
void hp03_Init(void)
//############################################################################
{
	// locals
	unsigned char ucValue;
	unsigned char i;

	/* use timer2 as 32768 HZ oszillator
	 * [1ms = (Comparewert+1) x Prescaler / fosc]
	 * fosc = 16 Mhz
	 * we use OC2 in toggle mode, therefore:
	 * f = 2* 32768 = 65536 Hz
	 * 16000000 / 65536 = 244,14 (with prescaler 1:1)
	*/

	OCR2 = 243;                        // CompareTime
	TCNT2 = 0;

	TCCR2   |= (1<<WGM01) | (0<<WGM00);             // CTC Mode 2
	TCCR2   |= (0<<CS02)  | (0<<CS01) | (1<<CS00); 	// 1:1 prescaler, Timer0 Start
	TCCR2   |= (0<<COM01) | (1<<COM00);				// Toggle OC Output on compare match

	// init IC interface
	i2c_init();

	// read calibration data
	SetChannel(DO_XCLR, OFF);

	i2c_start(EEPROM_ADDR + I2C_WRITE);
	i2c_write(0x10);
	i2c_start(EEPROM_ADDR + I2C_READ);

	// read 7 coefficents
	for(i = 0; i < 7; i++){
		ucValue = i2c_readAck();  	 	// read MSB
		uiCoeff[i] = (ucValue << 8);;
		ucValue = i2c_readAck();    	// read LSB
		uiCoeff[i] |= ucValue;
	}

	// read 4 parameters
	for(i = 0; i < 3; i++){
		ucParam[i] = i2c_readAck();
	}

	ucParam[3] = i2c_readNak();

	i2c_stop();

	return;
}

//############################################################################
void hp03_STM(void)
//############################################################################
{
	unsigned char ucValue = 0;
	unsigned int uiValue = 0;
	long a;
	float fdUT = 0.0;
	float fTemp = 0.0;
	float fPress = 0.0;

	// Set XCLR Output HIGH
	SetChannel(DO_XCLR, ON);

	// read temperature
	i2c_start(MEASVAL_ADDR + I2C_WRITE);
	i2c_write(0xFF);
	i2c_write(0xE8);
	i2c_stop();

	// wait some time
	for(a=0;a<200000;a++){asm("nop");}

	// read the value
	i2c_start(MEASVAL_ADDR + I2C_WRITE);
	i2c_write(0xFD);
	i2c_start(MEASVAL_ADDR + I2C_READ);

	ucValue = i2c_readAck();  	 	// read MSB
	uiValue  = (ucValue << 8);;
	ucValue = i2c_readNak();    	// read LSB
	uiValue  |= ucValue;

	i2c_stop();

	//
	if(uiValue >= uiCoeff[4]){
		fdUT = (float)uiValue - uiCoeff[4];
		fdUT = fdUT / (1 << 7);
		fdUT = fdUT * fdUT;
		fdUT = fdUT * ucParam[0];;
		fdUT = fdUT  / (1 << ucParam[2]);
		fdUT = (float)uiValue - uiCoeff[4] - fdUT;
	}else{
		fdUT = (float)uiValue - uiCoeff[4];
		fdUT = fdUT / (1 << 7);
		fdUT = fdUT * fdUT;
		fdUT = fdUT * ucParam[1];;
		fdUT = fdUT  / (1 << ucParam[2]);
		fdUT = (float)uiValue - uiCoeff[4] - fdUT;
	}
	fTemp = fdUT * uiCoeff[5];
	fTemp = fTemp / (1L << 16);
	fTemp = fTemp - (fTemp / (1 << ucParam[3]));
	fTemp = (250 + fTemp)/10;

	printf("T: %+5.1F C   -    P: ", (double)fTemp);

	// wait some time
	for(a=0;a<200000;a++){asm("nop");}

	// read pressure
	i2c_start(MEASVAL_ADDR + I2C_WRITE);
	i2c_write(0xFF);
	i2c_write(0xF0);
	i2c_stop();

	// wait some time
	for(a=0;a<200000;a++){asm("nop");}

	i2c_start(MEASVAL_ADDR + I2C_WRITE);
	i2c_write(0xFD);
	i2c_start(MEASVAL_ADDR + I2C_READ);

	ucValue = i2c_readAck();  	 	// read MSB
	uiValue = (ucValue << 8);;
	ucValue = i2c_readNak();    	// read LSB
	uiValue |= ucValue;

	i2c_stop();

	// Set XCLR Output LOW
	SetChannel(DO_XCLR, OFF);

	// calculate physical pressure vale
	// P = ((((C1 + C3 * dUT / (1<<10)) * 4) * (D1 - 7168)/(1<<14) - ((C2+(C4-1024)*dUT/(1<<14))*4)) * 10 /(1<<5)) + C7
	fPress = (float)uiCoeff[0] + (float)uiCoeff[2] * fdUT / (1<<10);
	fPress = fPress * (((float)uiValue - 7168)/(1<<14));
	fPress = fPress - ((float)uiCoeff[1]+((float)uiCoeff[3] - 1024)*fdUT/(1<<14))*4;
	fPress = fPress * 10 / (1<<5);
	fPress = fPress + (float)uiCoeff[6];

	printf("%5.1F hPa(abs) - ", (double)fPress/10);

	// calculate meteorologic pressure value (altitude and Temperature dependent)
	fPress = fPress * pow(((fTemp+273.15)/((fTemp+273.15)+ 3.0875) ), -5.255);

	printf(" %5.1F hPa(rel)\r", (double)fPress/10);

	return;
}

